/*
 * ar5315.c - AR2315/AR2316/AR2317/AR2318 specific system functions 
 *
 * copyright 2009 Sebastian Gottschall / NewMedia-NET GmbH / DD-WRT.COM
 * licensed under GPL conditions
 */
#include "mips32.c"
#include "spiflash.h"

/* definition of basic flash mappings */
static unsigned int sectorsize = 0x10000;
static unsigned int linuxaddr = 0xbfc10000;
static unsigned int flashbase = 0xa8000000;
static unsigned int flashsize = 0x800000;

#define AR2316_DSLBASE          0xB1000000	/* RESET CONTROL MMR */
#define AR531XPLUS_SPI              0xB1300000	/* SPI FLASH MMR */
#define AR2316_GPIO_DI          (AR2316_DSLBASE + 0x0088)
#define AR2316_RESET            (AR2316_DSLBASE + 0x0004)
#define AR2316_IF_CTL           (AR2316_DSLBASE + 0x0018)
#define AR2316_ENDIAN_CTL       (AR2316_DSLBASE + 0x000c)
#define AR2316_WDC              (AR2316_DSLBASE + 0x003c)
#define AR2316_AHB_ARB_CTL      (AR2316_DSLBASE + 0x0008)
#define RESET_WARM_WLAN0_MAC        0x00000001	/* warm reset WLAN0 MAC */
#define RESET_WARM_WLAN0_BB         0x00000002	/* warm reset WLAN0 BaseBand */
#define RESET_MPEGTS_RSVD           0x00000004	/* warm reset MPEG-TS */
#define RESET_PCIDMA                0x00000008	/* warm reset PCI ahb/dma */
#define RESET_MEMCTL                0x00000010	/* warm reset memory controller */
#define RESET_LOCAL                 0x00000020	/* warm reset local bus */
#define RESET_I2C_RSVD              0x00000040	/* warm reset I2C bus */
#define RESET_SPI                   0x00000080	/* warm reset SPI interface */
#define RESET_UART0                 0x00000100	/* warm reset UART0 */
#define RESET_IR_RSVD               0x00000200	/* warm reset IR interface */
#define RESET_EPHY0                 0x00000400	/* cold reset ENET0 phy */
#define RESET_ENET0                 0x00000800	/* cold reset ENET0 mac */

#define IF_TS_LOCAL                 2

#define CONFIG_ETHERNET             0x00000040	/* Ethernet byteswap */

#define ARB_CPU                     0x00000001	/* CPU, default */
#define ARB_WLAN                    0x00000002	/* WLAN */
#define ARB_MPEGTS_RSVD             0x00000004	/* MPEG-TS */
#define ARB_LOCAL                   0x00000008	/* LOCAL */
#define ARB_PCI                     0x00000010	/* PCI */
#define ARB_ETHERNET                0x00000020	/* Ethernet */
#define ARB_RETRY                   0x00000100	/* retry policy, debug only */

#define disable_watchdog() \
{ 					\
}					\

static int getGPIO(int nr)
{
	volatile unsigned int *gpio = (unsigned int *)AR2316_GPIO_DI;
	if ((*gpio & 1 << nr) == (1 << nr))
		return 1;
	return 0;
}

static void enable_ethernet(void)
{
	unsigned int mask = RESET_ENET0 | RESET_EPHY0;
	unsigned int regtmp;
	regtmp = sysRegRead(AR2316_AHB_ARB_CTL);
	regtmp |= ARB_ETHERNET;
	sysRegWrite(AR2316_AHB_ARB_CTL, regtmp);

	regtmp = sysRegRead(AR2316_RESET);
	sysRegWrite(AR2316_RESET, regtmp | mask);
	udelay(10000);

	regtmp = sysRegRead(AR2316_RESET);
	sysRegWrite(AR2316_RESET, regtmp & ~mask);
	udelay(10000);

	regtmp = sysRegRead(AR2316_IF_CTL);
	regtmp |= IF_TS_LOCAL;
	sysRegWrite(AR2316_IF_CTL, regtmp);

	regtmp = sysRegRead(AR2316_ENDIAN_CTL);
	regtmp &= ~CONFIG_ETHERNET;
	sysRegWrite(AR2316_ENDIAN_CTL, regtmp);
}


static const char *get_system_type(void)
{
	return "2315/6/7/8";
}
