/*
 * ar5312.c - AR5312/AR2313 specific system functions 
 *
 * copyright 2009 Sebastian Gottschall / NewMedia-NET GmbH / DD-WRT.COM
 * licensed under GPL conditions
 */

#include "mips32.c"

static unsigned int sectorsize = 0x10000;
static unsigned int linuxaddr = 0xbe010000;
static unsigned int flashbase = 0xbe000000;
static unsigned int flashsize = 0x800000;

#define AR531X_APBBASE  0xbc000000
#define AR531X_GPIO     (AR531X_APBBASE + 0x2000)
#define AR531X_GPIO_DI      (AR531X_GPIO + 0x04)
#define AR531X_RESETTMR (AR531X_APBBASE + 0x3000)
#define AR531X_WDC      (AR531X_RESETTMR + 0x0008)
#define AR531X_RESET    (AR531X_RESETTMR + 0x0020)
#define AR531X_ENABLE   (AR531X_RESETTMR + 0x0080)
#define ENABLE_ENET0              0x0002

#define RESET_ENET0          0x00000020	/* cold reset ENET0 mac */
#define RESET_EPHY0          0x00000008	/* cold reset ENET0 phy */

#define disable_watchdog() \
{ 					\
	sysRegWrite(AR531X_WDC, 0); 	\
}					\

static int getGPIO(int nr)
{
	volatile unsigned int *gpio = (unsigned int *)AR531X_GPIO_DI;
	if ((*gpio & 1 << nr) == (1 << nr))
		return 1;
	return 0;
}

static void enable_ethernet(void)
{
	unsigned int mask = RESET_ENET0 | RESET_EPHY0;
	unsigned int regtmp;

	regtmp = sysRegRead(AR531X_RESET);
	sysRegWrite(AR531X_RESET, regtmp | mask);
	udelay(15000);

	/* Pull out of reset */
	regtmp = sysRegRead(AR531X_RESET);
	sysRegWrite(AR531X_RESET, regtmp & ~mask);
	udelay(25);
	mask = ENABLE_ENET0;
	regtmp = sysRegRead(AR531X_ENABLE);
	sysRegWrite(AR531X_ENABLE, regtmp | mask);
}

static const char *get_system_type(void)
{
	return "5312";
}
